/****************************************************************************
 **
 **  Copyright (C) 2007, 2008 by Helge Kortendieck, alan@kasmithee.de
 **
 **  read/write vocabulary files
 **
 ** This file may be used under the terms of the GNU General Public
 ** License version 2.0 or later as published by the Free Software Foundation
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ****************************************************************************/

#include "rw.h"


rw::rw( QObject *parent ) : QObject( parent )
{
//  bNewDir = false;
}


rw::~rw()
{
}


void rw::readFileChk( const QString& sD, int iD )  // start reading file, setting things up
{
  sNf = sD;                                         // file name
  iPM = iD;                                         // mode: 0 = normal, 1 = flash card run
  bReading = true;
  errClear();
}

void rw::writeFileChk( const QString& sD, const QString& sD1, const QString& sD2 ) // start writing file, setting things up
{
  sNf = sD;                                     // file name
  sO = sD1;                                     // original language
  sT = sD2;                                     // translation language
  iPM = 0;                                      // program mode, 0 = normal, 1 = flash card, writing only in normal mode
  bReading = false;                             // writing
  errClear();
}

void rw::errClear()                             // clear error code and text
{
  iErr = 0;
  iWpCnt = 0;
  sErrTit = "";
  sErrTxt = "";
  fileChk();
}

void rw::fileChk()                              // check file, if technical any good, exists, readable, writeable
{
  QFileInfo qfi( sNf );

  if( !qfi.exists() ) {                             // does not exist = 4, needed for direct entering file name
    iErr = 4;
  } else if( !qfi.isReadable() ) {                  // is not readable = 2
    iErr = 2;
  } else if( !qfi.isWritable() ) {                  // is not writable = 1
    iErr = 1;
  }

  if( bReading == true ) {                      // *****  read mode
    if( iPM == 0 ) {                                // reading in normal mode, must be able to write
      if( iErr == 0 ) {
        read();
      }
    } else if( iPM == 1 ) {                         // reading flash card mode, do not need to write
      if( iErr < 2 ) {
        read();
      }
    } else {
      err();
    }
  } else {                                      // *****  write mode
    if( iErr == 0 ) {
      write();
    } else {
      err();
    }
  }
}

void rw::read()                                 // read/import file, evaluate in 'contentChk'
{
  slD.clear();                                 // clear file content QStringList
  QFile qf( sNf );
  if( !qf.open(QIODevice::ReadOnly | QIODevice::Text) ) {  // if we cannot open the file for reading, abort
    iErr = 8;                                   // undefined error while trying to open
    err();
  } else {
    QTextStream in(&qf);                        // initiate the stream to read from the playlist
    while( !in.atEnd() ) {                      // read the file (textstream)
      slD.append( in.readLine() );              // read line by line
    }
    qf.close();                                 // close voc file at EOF
  }

  if( slD.count() > 0 ) {                      // file contains lines
    iFwp = 0;                                     // extract languages from 1st line, if available
    sD = slD.at( 0 );
    if( sD.left( 10 ) == "Languages:" ) {         // found header description languages original and translation
      if( sD.count( "\t" ) == 1 ) {
        sO = "";
        sT = "";
        sD = sD.remove( 0, 10 );
        iD = sD.indexOf( "\t" );
        sO = sD.left( iD );
        sT = sD.remove( 0, iD + 1);             // *************************** check, + 1 or + 2 *******************************
      }
      iFwp = 1;                                   // 1st line has languages (might even be broken), ignore now
    } else {
      sO = "n a";
      sT = "n a";
    }

    bLinesInvalid = false;                        // invalid lines, # TABs != 1 or invalid wordpair
    sLinesInvalid = "";                           // file row # and content of invalid line(s)
    wordpair.clear();
    for( int i = iFwp; i < slD.count() ; ++i ) { // iterate over imported file and fill 'wordpair' struct
      sD = slD.at( i );
      if( sD.count( "\t" ) == 2 ) {                   // 2 TABs per line
        sD3 = "";
        sD4 = "";
        sD5 = "";
        iD = sD.indexOf( "\t" );
        sD3 = sD.left( iD );
        sD = sD.remove( 0, iD + 1 );
        iD = sD.indexOf( "\t" );
        sD4 = sD.left( iD );
        sD5 = sD.remove( 0, iD + 1 );
        if( sD3 != "" and sD4 != "" and sD5 != "" ) { // only append entries that have 'something' before and after the TAB
          wpD.orig = sD3;
          wpD.type = sD4;
          wpD.trans = sD5;
          wordpair.append( wpD );
          iWpCnt = iWpCnt + 1;
        } else {                                  // line without either orig, type or trans entry, skipped
          bLinesInvalid = true;
          sLinesInvalid = sLinesInvalid + QString::number( i + iFwp ) + ": " + sD + "\n";
          iErr = 2048;
        }
      } else {                                    // number of TABs != 2, invalid line, skipped
        bLinesInvalid = true;
        sLinesInvalid = sLinesInvalid + QString::number( i + iFwp ) + ": " + sD + "\n";
        iErr = 2048;
      }
    }

  } else if( slD.isEmpty() ) {                  // empty file ! can be a new one not yet filled
    iErr = 9;
    err();
    return;
  }

  if( wordpair.count() == 0 and iPM == 1 ) {    // no word pair in flash card mode
    iErr = 16;
  }

  sortWp();
  err();                                        // evaluate errors, called even if no error occurred
}

void rw::sortWp()                             // sort wordpair list
{
  if( wordpair.count() > 0 ) {                    // ***** start sorting *****
    wplD = wordpair;
    wordpair.clear();
    wordpair.append( wplD.at( 0 ) );              // init

    for( int i = 1; i < wplD.count(); ++i ) {
      sD = wplD.at( i ).orig;
      bIns = false;

      for( int i2 = wordpair.count() - 1; i2 > -1; --i2 ) {
        if( sD > wordpair.at( i2 ).orig ) {
          wordpair.insert( i2 + 1, wplD.at( i ) );
          bIns = true;
          break;
        }
      }

      if( bIns == false ) {                       // **************** needed ???? *****************
        wordpair.insert( 0, wplD.at( i ) );
      }
    }
  }
}


void rw::err()
{
  sErrTit = tr( "Non fatal read/write error" );
  if( iErr < 32 ) {
    sErrTit = tr( "Error reading file !" );
  } else if( iErr < 512 ) {
    sErrTit = tr( "Error writing file !" );
  }

  switch( iErr ) {
    case 0:                                           // no error
      if( bReading ) {
        emit sigReadOk( sO, sT );
      } else {
        emit sigWriteOk();
      }
      break;
    case 1:                                           // file not writeable
      sErrTxt = tr( "You do not have write permission for this file:\n" ) + sNf + 
                tr( "\nIt is pointless to continue, as any changes would be lost.\n \nAborting..." );
      if( bReading ) {
        if( iPM  == 0 ) {                             // normal mode abort
          emit sigReadAbort( sErrTit, sErrTxt, iErr );
        } else {                                      // flash card mode Ok
          emit sigReadOk( sO, sT );
        }
      } else {
        emit sigWriteAbort( sErrTit, sErrTxt, iErr );
      }
      break;
    case 2:                                           // file not readable
      sErrTxt = tr( "Sorry, could not open the file for reading:\n" ) + sNf + 
                tr( "\nThis could indicating a permission problem !\n \nAborting..." );
      if( bReading ) {
        emit sigReadAbort( sErrTit, sErrTxt, iErr );
      } else {
        emit sigWriteAbort( sErrTit, sErrTxt, iErr );
      }
      break;
    case 4:                                           // file does not exist
      sErrTxt = tr( "The specified file does not exist:\n" ) + sNf + 
                tr( "\n \nAborting..." );
      if( bReading ) {
        emit sigReadAbort( sErrTit, sErrTxt, iErr );
      } else {
        emit sigWriteAbort( sErrTit, sErrTxt, iErr );
      }
      break;
    case 8:                                           // error while opening file for read in text mode, should not happen?
      sErrTxt = tr( "Unexpected error while trying on open this file in text mode:\n" ) + sNf + 
                tr( "\n \nShould not happen !  Aborting..." );
      emit sigReadAbort( sErrTit, sErrTxt, iErr );
      break;
    case 9:                                           // error while opening file for read in text mode, should not happen?
      sErrTxt = tr( "The file\n" ) + sNf + 
                tr( "\nis empty !" );
      emit sigReadAbort( sErrTit, sErrTxt, iErr );
      break;
    case 16:                                          // empty file, no word pair for flash card run
      sErrTxt = tr( "There is no word pair in this vocabulary (?) file:\n" ) + sNf + 
                tr( "\n \nAborting..." );
      if( bReading ) {
        emit sigReadAbort( sErrTit, sErrTxt, iErr );
      } else {
        emit sigWriteAbort( sErrTit, sErrTxt, iErr );
      }
      break;
    case 32:                                          // writing: could not remove existing backup
      sErrTxt = tr( "Sorry, could not remove existing backup file\n" ) + sD + ".bak" + 
                tr( "\nThis would have been the first step in creating a backup\n"
                    "of the original version before saving the currently open\n"
                    "version as the new original.\n"
                    "This might be a file permission problem.\n \n"
                    "The currently open file was _NOT_ saved !\n \nAborting..." );
      if( !bReading ) {
        emit sigWriteAbort( sErrTit, sErrTxt, iErr );
      }
      break;
    case 64:                                          // writing: could not rename current voc file
      sErrTxt = tr( "Sorry, could not rename the file\n" ) + sD + 
                tr( "\n to\n" ) + sD + ".bak" + 
                tr( "\n \nThis would have been the second step in creating a backup\n"
                    "of the original version before saving the currently open\n"
                    "version as the new original.\n"
                    "This might be a file permission problem.\n \n"
                    "The currently open file was _NOT_ saved !\n \nAborting..." );
      if( !bReading ) {
        emit sigWriteAbort( sErrTit, sErrTxt, iErr );
      }
      break;
    case 128:                                         // writing: could not open voc file for write
    sErrTxt = tr( "Sorry, could not write the updated vocabulary file\n" ) + sD + 
              tr( "\nThis could indicate a problem with file permission\n"
                  "or disk is full.\n \n"
                  "Changes to this file were _NOT_ saved !\n \n" 
                  "However, a backup file exists:\n" ) + sD + ".bak" ;
      if( !bReading ) {
        emit sigWriteAbort( sErrTit, sErrTxt, iErr );
      }
      break;
    case 256:                                         // writing: could not rename current voc file
      sErrTxt = sErrTxt + tr( "\nUnexpectedly, I could not re-create the old file !\n" ) + sD + 
                          tr( "\nThe backup file _DOES_ exists, though !?" );
      if( !bReading ) {
        emit sigWriteAbort( sErrTit, sErrTxt, iErr );
      }
      break;
  }

  if( iErr < 512 ) {
    return;
  }

//  if( iErr == 0 or ( iPM == 1 and iErr < 2 ) ) {      // have non fatal error
  sErrTxt = tr( "File contains invalid lines !\n"
                "The following line(s) do either not have exactly 2 TABs (expected file structure is: origTABtypeTABtrans)\n"
                "or contain incomplete word pairs and have been skipped:\n \n") + sLinesInvalid;

  if( bReading ) {
    emit sigReadWErr( sErrTit, sErrTxt, iErr, sO, sT );
  } else {
    emit sigWriteWErr( sErrTit, sErrTxt, iErr );
  }
//  }
}


// ********************************************************************************
// **  writing file                                                              **
// ********************************************************************************
void rw::write()
{
  QFile f( sNf + ".bak" );                                // 1) delete old backup first
  if( f.exists() ) {
    if( !f.remove() ) {
      iErr = 32;                                            // cannot remove existing backup file
      err();
      return;
    }
  }

  QFile f2( sNf );                                        // 2) create backup of currently open file
  if( f2.exists() ) {
    if( !f2.rename( sNf + ".bak" ) ) {
      iErr = 64;                                            // cannot rename existing voc file
      err();
      return;
    }
  }

  QFile file( sNf );                                      // 3) write current file out
  if( file.open( QIODevice::WriteOnly ) ) {
    QTextStream stream( &file );
      stream << "Languages:" << sO << "\t" << sT << "\n";     // add header to top of list
      for (int i = 0; i < wordpair.size(); ++i) {
        stream << wordpair.at(i).orig << "\t" << wordpair.at(i).type << "\t" << wordpair.at(i).trans << "\n";
      }
    file.close();
    err();
  } else {
    iErr = 128;                                           // cannot open voc file for write
    err();
    QFile f3( sNf + ".bak" );                             // try to re-create old file
    if( !f3.copy( sNf ) ) {
      iErr = 256;                                         // cannot re-create old file (name) from *.bak version !?
      err();
    }
  }
}


#include "rw.moc"
