/****************************************************************************
 **
 **  Copyright (C) 2007, 2008 by Helge Kortendieck, alan@kasmithee.de
 **
 ** This file may be used under the terms of the GNU General Public
 ** License version 2.0 or later as published by the Free Software Foundation
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ****************************************************************************/

#ifndef _RW_H_
#define _RW_H_

#include "structs.h"

#include <math.h>

#include <QtAlgorithms>
#include <QFile>
#include <QFileInfo>
#include <QList>
#include <QObject>
#include <QString>
#include <QStringList>
#include <QTextStream>

class rw : public QObject
{
  Q_OBJECT

public:
  rw( QObject *parent = 0 );
     ~rw();

  int iErr;                 // error code
  int iWpCnt;               // number of word pairs separated by exactly 1 TAB

  QString sErrTit;          // error message titel
  QString sErrTxt;          // error message text
  QString sO;               // original language
  QString sT;               // translation language
  QList<WordPair> wordpair;

public slots:
    /*$PUBLIC_SLOTS$*/
  virtual void readFileChk( const QString & sD, int iD );               // check if file is readable/writeable
  virtual void writeFileChk( const QString & sD, const QString & sD1, const QString & sD2 );  // write file, list incl. lang header

  signals:
  void sigReadAbort( const QString& sErrTit, const QString& sErrTxt, int iErr );  // fatal errors while opening
  void sigReadWErr( const QString& sErrTit, const QString& sErrTxt, int iErr, const QString& sO, const QString& sT ); // errors while opening file
  void sigReadOk( const QString& sO, const QString& sT );  // successfully read file
  void sigWriteOk(); // file writing finished successfully
  void sigWriteWErr( const QString& sErrTit, const QString& sErrTxt, int iErr );  // non fatal error while writing file
  void sigWriteAbort( const QString& sErrTit, const QString& sErrTxt, int iErr ); // fatal error while writing file

protected:
    /*$PROTECTED_FUNCTIONS$*/

private:
  bool bDummy;
  bool bIns;                // used during sort, have we sorted in this item already?
  bool bReading;            // called to read file (bool == true) or to write file ( false )
  bool bLinesInvalid;       // flag whether files has invalid lines: # TABs != 1 or invalid wordpair

  int iD;
  int iD2;
  int iFwp;                 // line with 1st word pair
  int iPM;                  // program mode when in reading file mode, 0 = normal, 1 = flash card

  uint uiD;                 // uint dummy

  WordPair wpD;             // struct entity

  QList<WordPair> wplD;     // dummy wordpair list, used for sorting

  QString sD;               // QString dummy
  QString sD2;              // QString dummy
  QString sD3;              // QString dummy
  QString sD4;              // QString dummy
  QString sD5;              // QString dummy
  QString sDErr;            // dummy error string
  QString sLinesInvalid;    // file row # and content of invalid line(s)
  QString sNf;              // full path and name of file to open

  QStringList slD;
  QStringList slD2;
  QStringList slD3;

private slots:
  void err();               // collection function branch into errNo, errWith and errAbort from here
  void errClear();          // clear error code and text
  void fileChk();
  void read();              // read/import file
  void sortWp();            // sort wordpair list
  void write();             // write file, create backup first

  signals:

};

#endif // _RW_H_
