/****************************************************************************
 **
 **  Copyright (C) 2008 by Helge Kortendieck, alan@kasmithee.de
 **
 **  This file may be used under the terms of the GNU General Public
 **  License version 2.0 or later as published by the Free Software Foundation
 **
 **  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 **  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ****************************************************************************/

#ifndef SETTINGS_H
#define SETTINGS_H

#include <QApplication>
#include <QChar>
#include <QDialog>
#include <QDir>
#include <QEvent>
#include <QFileDialog>
#include <QFileInfo>
#include <QFont>
//#include <QHeaderView>
//#include <QKeyEvent>
#include <QList>
#include <QMessageBox>
//#include <QPixmap>
//#include <QPoint>
#include <QSettings>
#include <QSizePolicy>
#include <QString>
#include <QStringList>
#include <QTableWidget>
#include <QTableWidgetItem>
#include <QTimer>
#include <QWidget>

#include "structs.h"
#include "fcs.h"
#include "rw.h"
#include "ui_settings.h"

#include <iostream>
using namespace std;

class settings : public QDialog     // settings dialog
{
  Q_OBJECT

    public:
      settings(QDialog* parent = 0, Qt::WindowFlags fl = 0 );
        ~settings();

    protected:

    private:
      Ui::SettingsDlg uis;  // settings dialog widget

      QDialog wSetDlg;      // settings dialog widget

      bool bAutoStart;      // flag whether we should start flash card session when starting a desktop session
      bool bDictUse;        // use dict server

      bool bKde;                // KDE installed (.kde dir found)
      bool bKdeSet;             // set/unset qvtwfc.desktop in ./kde/Autostart
      bool bGnome;              // GNOME installed (.gnome2 dir found)
      bool bGnomeSet;           // set/unset qvtwfc.desktop in ./gnome2/Autostart   **********   assumption !!!
      bool bXfce4;              // xfce4 installed (.xfce4 dir found)               **********   assumption !!!
      bool bXfce4Set;           // set/unset qvtwfc.desktop in ./xfce4/Autostart    **********   assumption !!!

      int i;
      int i2;
      int iD;               // dummy int
      int iDictPort;        // port dict server is listening on, default = 2628
      int iErr;             // error code
      int iFontSize;        // font size, can be set in settings dialog
      int iIter;            // number of words to ask in test mode
      int iRun;             // enum, state of checkbox 'run at start of session' in settings dialog when toggled

      QList<DictDef> dictsD;
      QList<DictDef> dictsD2;
      QList<DictDef> dictsOnServer; // available dictionaries on the dict server
      QList<DictDef> dictsConf;     // dictionaries read from config
      QList<DictDef> dictsToQuery;  // dictionaries to be used (selected)
      DictDef dictdb;               // dummy item of dictionary list struct

      QString sD;           // dummy QString
      QString sD1;          // dummy QString
      QString sD2;          // dummy QString
      QString sDictIp;      // dict server Address/Ip, default = localhost
      QString sDictTT;      // tooltip text for dict status LED on main widget
      QString sDir;         // start dir for file dialogs
      QString sErrTit;      // error message titel
      QString sErrTxt;      // error message text
      QString sFileT;       // full path and name of voc file
      QString sFileTOrig;   //    full path and name of voc file when opening settings dialog

      QStringList slD;            // dummy QStringList

      QTableWidgetItem *qtwiD;    // dummy qtablewidgetitem

      fcs *sfcs;                      // flash card session, set/unset to start when starting a desktop session
      rw *rwfs;                       // read/write files, check files for settings dialog

    private slots:
      void readAbort( const QString& sErrTit, const QString& sErrTxt, int iErr );  // fatal error opening file
      void readOk( const QString& sD1, const QString & sD2 );                      // file successfully read, orig lang, trans lang, word list, # word pairs

      void setDlgOpen();              // open settings dialog
      void setDlgCancel();            // cancel clicked in settings dialog
      void setDlgChkFile();           // check test mode voc file: exists, readable ?
      void setDlgClose();             // close settings dialog
      void setDlgDictToggle();        // toggle state of dict frame: enabled/disabled
      void setDlgDictWDisable();      // disable dict widget in settings dialog
      void setDlgDictWEnable();       // enable dict widget in settings dialog (incl. clear)
      void setDlgDictErr();           // error while querying the dict server
      void setDlgDictPop();           // populate dict widget with all available dict servers, so that they can be selected
      void setDlgDisableSave();       // disable save button on first open, delayed to allow first dict server populating
      void setDlgEnableSave();        // enable save button
      void setDlgOpenFile();          // file dialog to select file to use in test mode run
      void setDlgQueryDictServer();   // query dict server for available dicts
      void setDlgReceiveAvailDicts( const QList<DictDef>&, const QList<DictDef>& ); // receive list with available dicts on dict server
      void setDlgRunTog( int iRun );  // run at start of session toggled
      void setDlgRunState( bool bKde, bool bKdeSet, bool bGnome, bool bGnomeSet, const QString& sErrTit, const QString& sErrTxt, int iErr ); // run at start of session toggled
      void setDlgSave();              // save in settings dialog clicked

      void settingsRead();
      void settingsWrite();

    signals:
      void sigSetDlgGetDicts( const QString& sD, int iD );  // get available dictionaries, dict IP & dict Port
      void sigSetDlgReReadSettings();                       // let main and dic re-read settings, after settings dialog save
      void sigSetDlgClose();
};

#endif 
